@@ -17,6 +17,10 @@ from .rcparams import rcParams
 STATIC_FILES = ("static/html/icons-svg-inline.html", "static/css/style.css")
 
 
+class BehaviourChangeWarning(Warning):
+    """Custom warning to ease filtering it."""
+
+
 def _check_tilde_start(x):
     return bool(isinstance(x, str) and x.startswith("~"))
 
@@ -326,6 +330,7 @@ class Numba:
     """A class to toggle numba states."""
 
     numba_flag = numba_check()
+    """bool: Indicates whether Numba optimizations are enabled. Defaults to False."""
 
     @classmethod
     def disable_numba(cls):
@@ -728,7 +733,10 @@ class Dask:
     """
 
     dask_flag = False
+    """bool: Enables Dask parallelization when set to True. Defaults to False."""
     dask_kwargs = None
+    """dict: Additional keyword arguments for Dask configuration.
+    Defaults to an empty dictionary."""
 
     @classmethod
     def enable_dask(cls, dask_kwargs=None):
